package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * 获取生产过程上下文对象接口【BUSI】
 */
public interface IMesProductionCustomContextStepService {

    @ApiOperation(value = "验证上下文工位加工锁是否存在")
    Boolean checkProductionLockIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文加工锁")
    Boolean dispatchProductionLockContext(StationRequestBean reqBean, String value);

    @ApiOperation(value = "删除上下文加工锁")
    void removeProductionLockContext(StationRequestBean reqBean);

    @ApiOperation(value = "发送设备质量信息")
    void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext);

    @ApiOperation(value = "存储上个扫描/读取信息")
    Boolean dispatchLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList);

    @ApiOperation(value = "获取上个扫描/读取信息")
    List<StationKvBean> getLastScanContext(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void sendStepContextMessage(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void sendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source);

    @ApiOperation(value = "发送工步内容")
    void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> resultList);

    @ApiOperation(value = "验证上下文班次班组信息是否存在")
    Boolean checkProdShiftDataIsExistContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "获取上下文班次班组信息")
    List<StationKvBean> getProdShiftDataContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "保存上下文班次班组信息", notes = "[JSON]List<StationKvBean>")
    Boolean dispatchProdShiftDataContext(String orgainzeCode, String workCenterCode, List<StationKvBean> prodShiftDatList);

    @ApiOperation(value = "删除上下文班次班组信息")
    void removeProdShiftDataContext(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "获取上下文工位加工数统计信息")
    List<StationKvBean> getProductionStatisticsContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文工位加工数统计信息", notes = "[JSON]List<StationKvBean>")
    Boolean dispatchProductionStatisticsContext(StationRequestBean reqBean, List<StationKvBean> prodShiftDatList);

    @ApiOperation(value = "新增上下文工位加工数统计信息")
    Integer dispatchProductionStatisticsContext(StationRequestBean reqBean, int count);

    @ApiOperation(value = "删除上下文工位加工数统计信息")
    void removeProductionStatisticsContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取生产线的当前班组班次信息")
    MesProdShiftContext getMesProdShiftKvBean(String orgainzeCode, String workCenterCode);

    @ApiOperation(value = "根据装配件规则ID获取可复用条码")
    String getRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode, String assemblyNosortCfgId);

    @ApiOperation(value = "根据设备代码获取可复用条码的个数")
    Integer getRepeatAssemblySnCount(String organizeCode, String workCenterCode, String workCellCode, String equipmentCode);

    @ApiOperation(value = "保存可复用条码到上下文")
    Boolean dispatchRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode, MesProductionAssemblyNosortContext productionAssemblyNosortContext);

    @ApiOperation(value = "删除上下文可复用条码信息")
    void removeRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode);

    @ApiOperation(value = "获取设备计数")
    Integer getShortCounter(StationRequestBean reqBean);

    @ApiOperation(value = "保存设备计数")
    void dispatchShortCounter(StationRequestBean reqBean, Integer shortCounter);

}
