package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdOrgExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesReworkTaskService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesTemplateService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNumberRuleMatchDispatchService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnExtService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblySortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskRequestModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTaskDetail;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.util.SpringContextsUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;


/**
 * @Description: 查询返工单
 * @Author: zxw
 * @Date: 2024/5/21 14:16
 * @Modify:
 */
@Service
@Slf4j
public class MesReworkTaskServiceImpl implements IMesReworkTaskService {

    private static final Logger LOGGER = LoggerFactory.getLogger(MesReworkTaskServiceImpl.class);


    @Autowired
    private MesReworkTaskRepository mesReworkTaskRepository;
    @Autowired
    private MesPartTypePictureRepository mesPartTypePictureRepository;

    @Autowired
    private MesPartRepository mesPartRepository;

    @Autowired
    private MesPartInspectionRepository mesPartInspectionRepository;

    @Autowired
    private MesPartInspectionDetailRepository mesPartInspectionDetailRepository;

    @Autowired
    private MesReworkTaskDetailRepository mesReworkTaskDetailRepository;

    @Autowired
    private MesProductionAssemblyRepository mesProductionAssemblyRepository;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private IMesTemplateService mesTemplateService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private MesMoveRepository moveRepository;

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRepository;

    @Autowired
    private IMesProdOrgExtService prodOrgExtService;

    @Autowired
    private IMesNumberRuleMatchDispatchService numberRuleMatchDispatchService;

    @Autowired
    private IMesProduceSnExtService produceSnExtService;

    @Override
    public ListPager<MesReworkTask> queryReworkTask(MesReworkTask mesReworkTask, Pager pager) {
        String organizeCode = !StringUtils.isEmpty(mesReworkTask.getOrganizeCode())?mesReworkTask.getOrganizeCode():AuthUtil.getOrganizeCode();
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        if (mesReworkTask.getSn() != null) {
            DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);
        }
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getReworkOrder(), "reworkOrder", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);

        pager = PagerHelper.getPager(pager, mesReworkTaskRepository.findByHqlWhereCount(packBean));

        List<MesReworkTask> resultList = mesReworkTaskRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }


    @Override
    public MesReworkTaskModel queryReworkTaskModel(MesReworkTaskRequestModel requestModel){
        /**
         * 获取返工单信息
         */
        MesReworkTask res = getMesReworkTask(requestModel);

        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(res);
        /**
         * 获取零件图片信息
         */
        MesPartTypePicture mesPartTypePicture = getMesPartTypePicture(res);
        /**
         * 获取缺陷记录
         */
        MesPartInspection mesPartInspection = getMesPartInspection(res);

        List<MesPartInspectionDetail> mesPartInspectionDetails = null;
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            /**
             * 获取零件缺陷明细列表
             */
            mesPartInspectionDetails = getMesPartInspectionDetail(reworkTaskDetails);
        }
        /**
         * 组装返回信息
         */
        MesReworkTaskModel mesReworkTaskModel = MesReworkTaskModel.builder()
                .reworkOrder(res.getReworkOrder())
                .sn(res.getSn())
                .mesPartTypePicture(mesPartTypePicture)
                .mesPartInspection(mesPartInspection)
                .mesPartInspectionDetails(mesPartInspectionDetails).build();


        return mesReworkTaskModel;
    }

    @Override
    public void reworkRecord(long detailId,String userName,String organizeCode) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(detailId, "partInspectionDetailId", packBean);

        List<MesReworkTaskDetail> mesReworkTaskDetailList = mesReworkTaskDetailRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesReworkTaskDetailList)) {
            MesPcnException.throwMesBusiException("返工单详情不存在");
        }
        mesReworkTaskDetailList.forEach(mesReworkTaskDetail->{
            mesReworkTaskDetail.setStatus(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_20.getValue());
            ConvertBean.serviceModelUpdate(mesReworkTaskDetail,userName);
        });
        mesReworkTaskDetailRepository.saveAll(mesReworkTaskDetailList);

    }


    @Override
    public List<MesProductionAssembly> assemblyQuery(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(requestModel.getCustSn(), "custSn", packBean);
        DdlPreparedPack.getNumEqualPack(requestModel.getIsOrigSn(), "isOrigSn", packBean);

        //DdlPreparedPack.getStringEqualPack(requestModel.getPartNo(), "partNo", packBean);
        if (!StringUtils.isEmpty(requestModel.getAssemblyPartNo())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getAssemblyPartNo(), "assemblyPartNo", packBean);
        }
        if (!Objects.isNull(requestModel.getAssemblyPartStatus())) {
            DdlPreparedPack.getNumEqualPack(requestModel.getAssemblyPartStatus(), "assemblyStatus", packBean);
        }

        List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
            MesPcnException.throwFlowException("装配件信息不存在");
        }
        return mesProductionAssemblies;
    }

    @Override
    public void reworkSuccess(MesReworkTaskRequestModel requestModel) {

        MesReworkTask mesReworkTask = getMesReworkTask(requestModel);
        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(mesReworkTask);
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            MesPcnException.throwFlowException("还有位置未返工完成,请检查");
        }
        if (MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue() == mesReworkTask.getType()) {
            mesReworkTask.setReworkedQty(requestModel.getReworkedQty());
            mesReworkTask.setScrapQty(requestModel.getScrapQty());
        } else {
            if (MesExtEnumUtil.NC_TYPE.REWORK.getValue() == requestModel.getType()) {
                mesReworkTask.setReworkedQty(mesReworkTask.getQty());
            } else if (MesExtEnumUtil.NC_TYPE.SCRAP.getValue() == requestModel.getType()) {
                //更新条码状态为报废
                updateMesProduceSn(requestModel);
                //更新返工数量
                mesReworkTask.setScrapQty(mesReworkTask.getQty());
            }
        }

        mesReworkTask.setStatus(MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue());
        ConvertBean.serviceModelUpdate(mesReworkTask,requestModel.getUserName());
        mesReworkTaskRepository.save(mesReworkTask);
    }

    @Override
    public MesProduceSnPrintModel doPrint(MesReworkTaskRequestModel requestModel) {
        String organizeCode = requestModel.getOrganizeCode();
        String userName = requestModel.getUserName();

        MesReworkTask mesReworkTask = getMesReworkTask(requestModel);
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(mesReworkTask);
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            MesPcnException.throwFlowException("还有位置未返工完成,请检查");
        }
        MesProduceSn sn = checkProduceSn(requestModel.getSn(), organizeCode);
        //查询物料信息
        MesPart mesPart = mesPartService.getMesPartByPartNo(sn.getPartNo(), organizeCode);
        String prodLabelTemplate = sn.getProdLabelTemplate();
        if (StringUtils.isEmpty(prodLabelTemplate)) {
            MesPcnException.throwMesBusiException("零件号[%s]的模板代码[%s]的模板信息丢失,请配置!", sn.getPartNo(), prodLabelTemplate);
        }
        MesProduceSnPrintModel mesProduceSnPrintModel = new MesProduceSnPrintModel();
        // 查模板代码
        MesLabelTemplate labelTemplate = mesTemplateService.getMesLabelTemplate(prodLabelTemplate,organizeCode);
        //根据反射获取策略类--封装打印数据
        String methodCode = labelTemplate.getMethodCode();
        //模板信息丢失抛出异常
        if (methodCode == null) {
            MesPcnException.throwMesBusiException("零件号[%s]的模板代码[%s]的反射类丢失,请配置!", sn.getPartNo(), prodLabelTemplate);
        }
        mesProduceSnPrintModel.setMesLabelTemplate(labelTemplate);
        mesProduceSnPrintModel.setProductSn(sn.getProductSn());
        mesProduceSnPrintModel.setPartNo(sn.getPartNo());
        mesProduceSnPrintModel.setOrganizeCode(organizeCode);
        List<MesProduceSn> snList = new ArrayList<>();
        sn.setDescription(String.format("源条码：%s",sn.getProductSn()));
        sn.setLotNo(TimeTool.getToday());
        sn.setProductSn(doGererateSerialNo(mesPart, organizeCode, requestModel.getUserName()));
        snList.add(sn);
        mesProduceSnPrintModel.getMesProduceSnList().addAll(snList);
        //发送新老条码关系给WMS
        createMove(requestModel.getSn(),sn.getProductSn(),organizeCode,requestModel.getUserName());
        //更新条码
        ConvertBean.serviceModelUpdate(sn,requestModel.getUserName());
        mesProduceSnRepository.update(sn);
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(requestModel.getSn(),"productSn",packBean);
        DdlPreparedPack.getStringEqualPack(sn.getPartNo(),"partNo",packBean);
        //更新加工记录
        mesProductionRecordRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime","productSn","description"},
                new Object[]{userName, TimeTool.getNowTime(true),sn.getProductSn(),sn.getDescription()},packBean);
        //更新装配记录
        mesProductionAssemblyRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime","productSn","description"},
                new Object[]{userName, TimeTool.getNowTime(true),sn.getProductSn(),sn.getDescription()},packBean);
        //更新返工记录
        packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(requestModel.getSn(),"sn",packBean);
        DdlPreparedPack.getStringEqualPack(sn.getPartNo(),"partNo",packBean);
        mesReworkTaskRepository.updateByProperties(new String[]{"modifyUser", "modifyDatetime","sn","description"},
                new Object[]{userName, TimeTool.getNowTime(true),sn.getProductSn(),sn.getDescription()},packBean);
        IPrintTemplateStrategyService strategyService = (IPrintTemplateStrategyService) SpringContextsUtil.getBean(methodCode);
        return strategyService.execute(null, mesProduceSnPrintModel, null, null, null, true);
    }

    @Override
    public void doAssemblySnRepeatByScan(MesReworkTaskRequestModel requestModel) {
        boolean checkResult = true;
        //查询装配记录
        requestModel.setAssemblyPartStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
        List<MesProductionAssembly> productionAssemblies = assemblyQuery(requestModel).stream().filter(t -> CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == t.getIsOrigSn()).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(productionAssemblies)) MesPcnException.throwFlowException("未查询到没有替换的装配件记录");
        Map<String,MesWorkCenter> mesWorkCenterMap = new HashMap<>();
        //匹配装配件
        for (MesProductionAssembly assembly : productionAssemblies) {
            Object context = getContext(requestModel, assembly,mesWorkCenterMap);
            //匹配规则
            Map<String, Object> result = numberRuleMatchDispatchService.matchNumberRule(requestModel.getOrganizeCode(), requestModel.getSn(), context);
            //匹配成功
            if ((Boolean) result.get(MesPcnExtConstWords.RESULT)){
                // 需要变更之前的记录
                assembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_20.getValue());

                // 需要新增一条替换条码的记录
                MesProductionAssembly mesProductionRepeatAssembly = new MesProductionAssembly();
                BeanUtils.copyProperties(assembly, mesProductionRepeatAssembly);
                mesProductionRepeatAssembly.setAssemblySn(requestModel.getSn());
                mesProductionRepeatAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
                mesProductionRepeatAssembly.setId(null);
                mesProductionRepeatAssembly.setIsOrigSn(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                mesProductionRepeatAssembly.setFid(UUID .randomUUID().toString());
                ConvertBean.serviceModelInitialize(mesProductionRepeatAssembly,requestModel.getUserName());

                assembly.setRepeatAssemblySn(requestModel.getSn());
                // 原装配件记录为已解绑
                ConvertBean.serviceModelUpdate(assembly,requestModel.getUserName());
                mesProductionAssemblyRepository.update(assembly);
                // 新装配件记录为已装配
                mesProductionAssemblyRepository.insert(mesProductionRepeatAssembly);
                //自制件更新条码状态
                if(MesExtEnumUtil.ASSEMBLY_MATCH_TYPE.MATCH_TYPE_10.getValue() == assembly.getMatchType()){
                    List<MesProduceSn> produceSnList = produceSnExtService.getProduceSnList(requestModel.getOrganizeCode(), requestModel.getSn());
                    if (CollectionUtils.isEmpty(produceSnList)) MesPcnException.throwFlowException("零件条码[%s]信息不存在!", requestModel.getSn());
                    Optional<MesProduceSn> optional = produceSnList.stream().filter(sn->sn.getId().equals(((MesProductionAssemblyContext)context).getProductSnId())).findFirst();
                    if (!optional.isPresent()) MesPcnException.throwFlowException("零件条码[%s]信息不存在!", requestModel.getSn());
                    // 更新新替换得条码为已装配
                    updateNewSn(optional.get(),requestModel.getUserName());
                    //更新原条码状态为已拆解
                    updateOldSn(assembly);
                }
                checkResult = false;
                break;
            }
        }
        if(checkResult) MesPcnException.throwFlowException("装配件替换失败，未匹配到符合规则的已装配数据");
        /**
         * 需要更新NC标记后进入的表明细状态为已处理
         */
        for (String detailId : requestModel.getDetailIds().split(",")) {
            reworkRecord(Long.parseLong(detailId),requestModel.getUserName(),requestModel.getOrganizeCode());
        }
    }

    private Object getContext(MesReworkTaskRequestModel requestModel, MesProductionAssembly mesProductionAssembly,Map<String,MesWorkCenter> mesWorkCenterMap) {
        if(StringUtils.isEmpty(mesProductionAssembly.getWorkCenterCode())) MesPcnException.throwFlowException("产线代码不存在");
        Object context =  null;
        MesWorkCenter workCenterDb = null;
        //产线类型
        if(!mesWorkCenterMap.containsKey(mesProductionAssembly.getWorkCenterCode())){
            workCenterDb = prodOrgExtService.getWorkCenterDb(requestModel.getOrganizeCode(), mesProductionAssembly.getWorkCenterCode());
            if(Objects.isNull(workCenterDb)) MesPcnException.throwFlowException("产线【%s】信息不存在", mesProductionAssembly.getWorkCenterCode());
            mesWorkCenterMap.put(mesProductionAssembly.getWorkCenterCode(),workCenterDb);
        }else{
            workCenterDb = mesWorkCenterMap.get(mesProductionAssembly.getWorkCenterCode());
        }
        if (MesExtEnumUtil.WORK_CENTER_TYPE.SORT.getValue() == workCenterDb.getCenterType())  context = new MesProductionAssemblySortContext().copy(mesProductionAssembly);
        else context = new MesProductionAssemblyNosortContext().copy(mesProductionAssembly);
        return context;
    }

    private String doGererateSerialNo(MesPart part,String organizeCode,String userName) {
        if(StringUtils.isEmpty(part.getProductMatchRule())) MesPcnException.throwMesBusiException("请检查零件信息,零件[%s]信息未维护零件条码编码规则!", part.getPartNo());
        return syncFuncService.syncSerialNo(
                new GenSerialNoModel(part.getProductMatchRule()).partNo(part.getPartNo()).putDataMap(MesPart.class.getSimpleName(), part).organizeCode(organizeCode),
                userName, organizeCode, 1).getResultList().get(0).toString();
    }

    @Override
    public boolean validateSn(MesReworkTaskRequestModel requestModel) {
        //
        List<String> snList = Arrays.asList(requestModel.getSn().split(","));
        List<String> idList = Arrays.asList(requestModel.getAssemblyIds().split(","));
        // 判断数量是否一致
        if (snList.size() != idList.size()) MesPcnException.throwFlowException("选中的装配件个数和输入的个数不一致");
        // 判断顺序是否一致
        Map<String, String> map = IntStream.range(0, snList.size()).boxed().collect(Collectors.toMap(idList::get, snList::get, (a, b) -> b));
        List<Long> idLonglist = idList.stream().map(str -> Long.parseLong(str)).collect(Collectors.toList());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(idLonglist, "id", ddlPackBean);
        List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
            MesPcnException.throwFlowException("装配件记录不存在");
        }
        Map<Long, String> mapProduction = mesProductionAssemblies.stream().collect(Collectors.toMap(mesProductionAssembly -> mesProductionAssembly.getId(), mesProductionAssembly -> mesProductionAssembly.getPartNo()));
        map.forEach((k, v) -> {
            MesProductionAssembly mesProductionAssembly = mesProductionAssemblyRepository.getById(Long.parseLong(k));
            if (!Objects.equals(mapProduction.get(k), mesProductionAssembly.getPartNo())) {
                MesPcnException.throwFlowException(String.format("装配件记录不匹配id=【%s】,sn=【%s】", k, v));
            }
        });

        return false;
    }

    /**
     * map id和sn对应关系
     * mapProductionAssemble  id 和物料对应关系
     * mapProduceSn 物料和条码对应关系
     *
     * 根据mapProductionAssemble id找map的sn
     * 根据mapProductionAssemble 物料找mmapProduceSn的sn
     * 两者必须一致才允许
     *
     * @param requestModel
     */
    @Override
    public void doAssemblySnRepeat(MesReworkTaskRequestModel requestModel) {

        try {
            //
            List<String> snList = Arrays.asList(requestModel.getSn().split(","));
            snList = snList.stream().distinct().collect(Collectors.toList());
            List<String> idList = Arrays.asList(requestModel.getAssemblyIds().split(","));
            // 判断数量是否一致
            if (snList.size() != idList.size()) MesPcnException.throwFlowException("选中的装配件个数和输入的个数不一致");
            // 判断顺序是否一致  key = id, value = sn
            Map<String, String> map = IntStream.range(0, snList.size()).boxed().collect(Collectors.toMap(idList::get, snList::get, (a, b) -> b));
            List<Long> idLonglist = idList.stream().map(str -> Long.parseLong(str)).collect(Collectors.toList());

            // 获取装配件记录列表
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getInPackList(idLonglist, "id", ddlPackBean);
            List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
                MesPcnException.throwFlowException("装配件记录不存在");
            }
            Map<Long, String> mapProductionAssemble = mesProductionAssemblies.stream().collect(Collectors.toMap(mesProductionAssembly -> mesProductionAssembly.getId(), mesProductionAssembly -> mesProductionAssembly.getAssemblyPartNo()));

            List<Integer> snStatusList = Stream.of(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue(), MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY_BACK.getValue()).collect(Collectors.toList());
            // 获取替换条码信息列表
            DdlPackBean snDdlPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getInPackList(snList, "productSn", snDdlPackBean);
            DdlPreparedPack.getInPackList(snStatusList, "snStatus", snDdlPackBean);

            List<MesProduceSn> mesProduceSnList = mesProduceSnRepository.findByHqlWhere(snDdlPackBean);
            if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
                MesPcnException.throwFlowException("零件条码不存在");
            }
            List<MesProduceSn> mesProduceSns =  new ArrayList<>();
            Map<String, List<MesProduceSn>>  productSnListMap = mesProduceSnList.stream().collect(Collectors.groupingBy(mesProduceSn -> mesProduceSn.getProductSn()));
            Map<String, MesProduceSn> productSnMap = new HashedMap();
            productSnListMap.forEach((k,v) -> {
                MesProduceSn produceSn = v.stream().sorted(Comparator.comparing(MesProduceSn::getCreateDatetime).reversed()).findFirst().get();
                mesProduceSns.add(produceSn);
                productSnMap.put(produceSn.getProductSn(), produceSn);
            });

            Map<String, String> mapProduceSn = mesProduceSns.stream().collect(Collectors.toMap(mesProduceSn -> mesProduceSn.getPartNo(), mesProduceSn -> mesProduceSn.getProductSn()));

            mapProductionAssemble.forEach((k, v) -> {
                String sn = map.get(k + "");
                String newSn = mapProduceSn.get(v);
                if (!Objects.equals(sn, newSn)) {
                    MesPcnException.throwFlowException(String.format("装配件记录不匹配id=【%s】,sn=【%s】", k, sn));
                }
            });

            for (String sn : snList) {
                MesProduceSn mesProduceSn = productSnMap.get(sn);
                if (mesProduceSn == null) {
                    MesPcnException.throwFlowException("条码信息不存在");
                }
                requestModel.setAssemblyPartNo(mesProduceSn.getPartNo());
                requestModel.setAssemblyPartStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
                List<MesProductionAssembly> mesProductionAssemblieList = assemblyQuery(requestModel);
                if (CollectionUtils.isEmpty(mesProductionAssemblieList)) {
                    MesPcnException.throwFlowException("未匹配到该条码的零件号");
                }
                // 需要变更之前的记录
                MesProductionAssembly mesProductionAssembly = mesProductionAssemblieList.get(0);
                mesProductionAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_20.getValue());


                // 需要新增一条替换条码的记录
                MesProductionAssembly mesProductionRepeatAssembly = new MesProductionAssembly();
                BeanUtils.copyProperties(mesProductionAssembly, mesProductionRepeatAssembly);
                mesProductionRepeatAssembly.setAssemblySn(sn);
                mesProductionRepeatAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
                mesProductionRepeatAssembly.setId(null);
                mesProductionRepeatAssembly.setIsOrigSn(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                mesProductionRepeatAssembly.setFid(UUID .randomUUID().toString());

                mesProductionAssembly.setRepeatAssemblySn(sn);
                // 更新新替换得条码为已装配
                updateNewSn(mesProduceSn,requestModel.getUserName());
                // 更新原条码状态为已拆解
                ConvertBean.serviceModelUpdate(mesProductionAssembly,requestModel.getUserName());
                updateOldSn(mesProductionAssembly);
                // 原装配件记录为已解绑
                mesProductionAssemblyRepository.update(mesProductionAssembly);
                // 新装配件记录为已装配
                ConvertBean.serviceModelInitialize(mesProductionRepeatAssembly,requestModel.getUserName());
                mesProductionAssemblyRepository.insert(mesProductionRepeatAssembly);
            }
            /**
             * 需要更新NC标记后进入的表明细状态为已处理
             */
            for (String detailId : requestModel.getDetailIds().split(",")) {
                reworkRecord(Long.parseLong(detailId),requestModel.getUserName(),requestModel.getOrganizeCode());
            }
        } catch (Exception e) {
            log.error("替换条码异常", e);
            throw e;
        }

    }

    private void updateNewSn(MesProduceSn mesProduceSn,String userName) {
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY.getValue());
        ConvertBean.serviceModelUpdate(mesProduceSn, userName);
        mesProduceSnRepository.update(mesProduceSn);
    }

    private void updateOldSn(MesProductionAssembly mesProductionAssembly) {
        DdlPackBean oldSnPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(mesProductionAssembly.getAssemblySn(), "productSn", oldSnPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, oldSnPackBean);

        MesProduceSn oldProduceSn = mesProduceSnRepository.getByProperty(oldSnPackBean);
        if (oldProduceSn == null) {
            MesPcnException.throwFlowException(String.format("原条码信息不存在【%s】", mesProductionAssembly.getProductSn()));
        }

        ConvertBean.serviceModelUpdate(oldProduceSn, mesProductionAssembly.getModifyUser());
        oldProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY_BACK.getValue());
        mesProduceSnRepository.update(oldProduceSn);
    }


    private MesPartInspection getMesPartInspection(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);

        MesPartInspection mesPartInspection = mesPartInspectionRepository.getByProperty(packBean);

        return mesPartInspection;
    }

    private List<MesPartInspectionDetail> getMesPartInspectionDetail(List<MesReworkTaskDetail> reworkTaskDetails) {

        List<Long> ids = reworkTaskDetails.stream().map(MesReworkTaskDetail::getPartInspectionDetailId).collect(Collectors.toList());

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reworkTaskDetails.get(0).getOrganizeCode());
        DdlPreparedPack.getInPackList(ids, "id", packBean);

        return mesPartInspectionDetailRepository.findByHqlWhere(packBean);
    }

    private MesPartTypePicture getMesPartTypePicture(MesReworkTask mesReworkTask) {
        DdlPackBean partBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getPartNo(), "partNo", partBean);
        MesPart mesPart = mesPartRepository.getByProperty(partBean);
        if (mesPart == null) {
            MesPcnException.throwFlowException(String.format("零件信息不存在,根据物料号查询[%s]", mesReworkTask.getPartNo()));
        }

        DdlPackBean pictureBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPart.getPartTypeCode(), "partTypeCode", pictureBean);

        return mesPartTypePictureRepository.getByProperty(pictureBean);
    }

    private MesReworkTask getMesReworkTask(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        if (!StringUtils.isEmpty(requestModel.getSn())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getSn(), "sn", packBean);
        }
        if (!StringUtils.isEmpty(requestModel.getReworkOrder())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getReworkOrder(), "reworkOrder", packBean);
        }
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);

        MesReworkTask res = mesReworkTaskRepository.getByProperty(packBean);
        if (res == null) {
            throw new ImppBusiException("返工单信息不存在");
        }
        return res;
    }
    private List<MesReworkTaskDetail> getMesReworkTaskDetail(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesReworkTask.getId(), "pid", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_10.getValue(), "status", packBean);

        return mesReworkTaskDetailRepository.findByHqlWhere(packBean);
    }

    private void updateMesProduceSn(MesReworkTaskRequestModel requestModel) {
        MesProduceSn produceSn = checkProduceSn(requestModel.getSn(), requestModel.getOrganizeCode());
        produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.SCRAP.getValue());
        ConvertBean.serviceModelUpdate(produceSn,requestModel.getUserName());
        mesProduceSnRepository.update(produceSn);
    }

    private MesProduceSn checkProduceSn(String serialNumber, String org) {
        //根据扫描的条码查询条码是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(serialNumber, "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = mesProduceSnRepository.getByProperty(packBean);

        //若不存在则提示条码不存在
        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此条码不存在，请检查数据", serialNumber));
        }

        return produceSn;
    }

    private MesMove createMove(String oldSn, String sn, String organizeCode, String userName) {
        MesMove move = new MesMove();
        move.setOrganizeCode(organizeCode);
        move.setFactoryCode(organizeCode);
        move.setPostDate(TimeTool.getToday());
        move.setPostTime(TimeTool.getTimeShortWithColon());
        move.setMoveType(MesExtEnumUtil.MOVE_TYPE.REWORK_REPLACE.getValue());
        move.setProductSn(sn);
        move.setProductSnOld(oldSn);
        ConvertBean.serviceModelInitialize(move, userName);
        return moveRepository.insert(move);
    }
}
